/****************************************************************************
 Creation 13/06/2003                                               by Sylvain


                            CODEC.C
                            --------


 Rev 13/06/2003                                 Copyright (C) BoostWorks 2003 
*****************************************************************************

         Encode/decode using a xor and a key

****************************************************************************/
static char *rcsid="$Id: codec.c,v 1.2 2003/06/23 13:38:23 sylvain Exp $";
static long int use_rcsid = sizeof(rcsid) + (long int)&use_rcsid;

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "codec.h"


typedef struct _tagCoDecContextInternal
{
    unsigned char * pKey;
    int cbKey;
} CoDecContextInternal;



/*---------------------------------------------------------------------------
**
**        CODECINIT
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --CoDecInit--
**
**	Set key to use for encoding/decoding
**
**  ARGUMENTS
**        Argument1:  unsigned char * pKey
**        Argument2: int cbKey
**  RETOUR/RESULTAT
**        CODECCONTEXT * 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 13/06/2003 
**--------------------------------------------------------------------------*/
CODECCONTEXT * CoDecInit( unsigned char * pKey, int cbKey )
{
    CoDecContextInternal * pContextInternal;

    if( pKey == NULL ) return pKey;
    if( cbKey <= 0 ) cbKey = strlen(pKey);
    pContextInternal = (CoDecContextInternal *)calloc( 1, sizeof(CoDecContextInternal) );
    if( pContextInternal == NULL) return pContextInternal;
    pContextInternal->pKey = (unsigned char *)malloc( cbKey );
    if( pContextInternal->pKey == NULL )
    {
	CoDecDestroy( pContextInternal );
	return NULL;
    }
    pContextInternal->cbKey = cbKey;
    memcpy( pContextInternal->pKey, pKey, cbKey );
    
    return (CODECCONTEXT *)pContextInternal;
}

/*---------------------------------------------------------------------------
**
**        CODECENCODEBYTES
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --CoDecEncodeBytes--
**
**	Encode string (currently using a xor)
**
**  ARGUMENTS
**        Argument1:  CODECCONTEXT * pContext
**        Argument2: unsigned char * pIn
**        Argument3: unsigned char * pOut
**        Argument4: int cb
**  RETOUR/RESULTAT
**        int 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 13/06/2003 
**--------------------------------------------------------------------------*/
int CoDecEncodeBytes( CODECCONTEXT * pContext, unsigned char * pIn, unsigned char * pOut, int cb )
{
    CoDecContextInternal * pContextInternal = (CoDecContextInternal *)pContext;
    int i;
    if( pContextInternal == NULL ) return -1;
    if( pIn == NULL || pOut == NULL || cb <= 0 ) return -1;

    for( i = 0; i < cb; i ++ )
    {
	pOut[i] = pIn[i] ^ pContextInternal->pKey[i % pContextInternal->cbKey];
    }
    return 0;
}


/*---------------------------------------------------------------------------
**
**        CODECDECODEBYTES
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --CoDecDecodeBytes--
**
**	Decode string
**
**  ARGUMENTS
**        Argument1:  CODECCONTEXT * pContext
**        Argument2: unsigned char * pIn
**        Argument3: unsigned char * pOut
**        Argument4: int cb
**  RETOUR/RESULTAT
**        int 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 13/06/2003 
**--------------------------------------------------------------------------*/
int CoDecDecodeBytes( CODECCONTEXT * pContext, unsigned char * pIn, unsigned char * pOut, int cb )
{
    return CoDecEncodeBytes( pContext, pIn, pOut, cb );
}


/*---------------------------------------------------------------------------
**
**        CODECDESTROY
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --CoDecDestroy--
**
**	Free context
**
**  ARGUMENTS
**        Argument1: CODECCONTEXT * pContext
**  RETOUR/RESULTAT
**        int 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 13/06/2003 
**--------------------------------------------------------------------------*/
int CoDecDestroy( CODECCONTEXT * pContext )
{
    CoDecContextInternal * pContextInternal = (CoDecContextInternal *)pContext;
    if( pContextInternal == NULL ) return -1;
    if( pContextInternal->pKey ) free( pContextInternal->pKey );
    free( pContextInternal );
    return 0;
}

#ifdef MAIN
void main()
{
    char szText[] = "coucou me voila - ceci est un test - encore un - y'en a jamais de trop";
    unsigned char szResult[256];
    char szResult2[256];
    CODECCONTEXT * pContext;

    pContext = CoDecInit( "qsdfmlkqslmdf5646882117", -1 );

    CoDecEncodeBytes( pContext, szText, szResult, strlen(szText) + 1); 
    CoDecEncodeBytes( pContext, szResult, szResult2, strlen(szText) + 1); 
    printf( "%s\n", szResult2 );
    CoDecDestroy( pContext );
}
#endif