/****************************************************************************
 Creation 04/08/2003                                               by Sylvain


                           CODECPWD.C
                           ----------


 Rev 04/08/2003                                 Copyright (C) BoostWorks 2003 
*****************************************************************************

  
****************************************************************************/
static char *rcsid="$Id$";
static long int use_rcsid = sizeof(rcsid) + (long int)&use_rcsid;

#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "codecpwd.h"
#include "base64.h"

#define USE_CRYPTOAPI 0
#if USE_CRYPTOAPI

#include "crypto2.h"

#else

#include "codec.h"

static unsigned char szKey[] = "g`/qsfm-fr,jz~4t|dlx,>].[{*&-K";
static CODECCONTEXT * s_pContext;
#endif

// Statics vars

/*---------------------------------------------------------------------------
**
**        ENCRYPTPASSWORD
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --EncryptPassword--
**  ARGUMENTS
**        Argument1:  LPCTSTR pszPasswordToEncript
**        Argument2: LPCTSTR * ppszEncriptedPassword
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 04/08/2003 
**--------------------------------------------------------------------------*/
BOOL EncryptPassword( LPCTSTR pszPasswordToEncript, LPCTSTR * ppszEncriptedPassword )
{
    TCHAR * pszCryptoEncoded;
    TCHAR * pszCryptoAndBase64Encoded;
    INT cbLen;

    *ppszEncriptedPassword = NULL;
    if( ! pszPasswordToEncript ) 
    {
	return FALSE;
    }
    
    cbLen = _tcslen(pszPasswordToEncript) + 1;
    pszCryptoEncoded = (TCHAR *)calloc( 1, cbLen * sizeof(TCHAR) );
    if( ! pszCryptoEncoded )
    {
	return FALSE;
    }

#if USE_CRYPTOAPI    
    if( ! EncryptString( pszPasswordToEncript, (BYTE *)pszCryptoEncoded, cbLen ) )
    {
	return FALSE;
    }
#else
    if( s_pContext == NULL )
    {
	s_pContext = CoDecInit( szKey, -1 );
    }
    if( s_pContext == NULL )
    {
	free(pszCryptoEncoded);
	return FALSE;
    }
    if( CoDecEncodeBytes( s_pContext, (unsigned char * )pszPasswordToEncript, 
		(unsigned char * )pszCryptoEncoded, cbLen ) < 0 )

    {
	free(pszCryptoEncoded);
	return FALSE;
    }
#endif

    pszCryptoAndBase64Encoded = (TCHAR *)calloc( 1, cbLen * sizeof(TCHAR) * 4 );
    if( ! pszCryptoAndBase64Encoded ) 
    {
	free(pszCryptoEncoded);
	return FALSE;
    }

    base64_encode_bin( pszCryptoEncoded, cbLen, pszCryptoAndBase64Encoded, cbLen * 4, NULL );
    free(pszCryptoEncoded);
    *ppszEncriptedPassword = pszCryptoAndBase64Encoded;
    return TRUE;
}


/*---------------------------------------------------------------------------
**
**        DECRYPTPASSWORD
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --DecryptPassword--
**  ARGUMENTS
**        Argument1:  LPCTSTR pszEncriptedPassword
**        Argument2: LPCTSTR * ppszDecriptedPassword
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 04/08/2003 
**--------------------------------------------------------------------------*/
BOOL DecryptPassword( LPCTSTR pszEncriptedPassword, LPCTSTR * ppszDecriptedPassword )
{
    TCHAR * pszBase64Decoded;
    TCHAR * pszCryptoDecoded;
    INT cbLen;

    *ppszDecriptedPassword = NULL;
    if( ! pszEncriptedPassword )
    {
	return FALSE;
    }

    cbLen = _tcslen(pszEncriptedPassword) + 1;
    pszBase64Decoded = (TCHAR *)calloc( 1, cbLen * sizeof(TCHAR) );
    if( ! pszBase64Decoded )
    {
	return FALSE;
    }

    base64_decode(pszEncriptedPassword, pszBase64Decoded, cbLen );

    pszCryptoDecoded = (TCHAR *)calloc( 1, cbLen * sizeof(TCHAR) );
    if( ! pszCryptoDecoded )
    {
	free( pszBase64Decoded );
	return FALSE;
    }

#if USE_CRYPTOAPI
    if( ! DecryptString( (BYTE *)pszBase64Decoded, cbLen  * sizeof(TCHAR), pszCryptoDecoded, cbLen ) )
    {
	free( pszBase64Decoded );
	free( pszCryptoDecoded );
	return FALSE;
    }
#else
    if( s_pContext == NULL )
    {
	s_pContext = CoDecInit( szKey, -1 );
    }
    if( s_pContext == NULL )
    {
	free( pszBase64Decoded );
	return FALSE;
    }
    
    if( CoDecDecodeBytes( s_pContext, (unsigned char * )pszBase64Decoded, 
		(unsigned char * )pszCryptoDecoded, cbLen ) < 0 )
    {
	free( pszBase64Decoded );
	return FALSE;
    }
#endif

    free( pszBase64Decoded );
    *ppszDecriptedPassword = pszCryptoDecoded;
    return TRUE;
}


/*---------------------------------------------------------------------------
**
**        FREEPASSWORDBUFFER
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --FreePasswordBuffer--
**  ARGUMENTS
**        Argument1: LPCTSTR psz
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 04/08/2003 
**--------------------------------------------------------------------------*/
BOOL FreePasswordBuffer( LPCTSTR psz )
{
    if( psz ) free( (void *)psz );
    return TRUE;
}


#if 1
void main( int argc, char * argv[])
{
    TCHAR szPasswordToEncript[] = "coucou";
    TCHAR * pszPasswordToEncript= szPasswordToEncript;
    LPCTSTR pszEncriptedPassword;
    LPCTSTR pszDecriptedPassword;

    if( argc == 2 )
    {
	pszPasswordToEncript = argv[1];
    }
    
    printf( "To encrypt : %s\n", pszPasswordToEncript );
    if( ! EncryptPassword( pszPasswordToEncript, &pszEncriptedPassword ) )
    {
	printf( "EncryptPassword() error\n" );
	return;
    }
    printf( "Encrypted : %s\n", pszEncriptedPassword );

    if( ! DecryptPassword( pszEncriptedPassword, &pszDecriptedPassword ) )
    {
	printf( "DecryptPassword() error\n" );
	return;
    }
    printf( "Decrypted : %s\n", pszDecriptedPassword );
    
    FreePasswordBuffer( pszEncriptedPassword );
    FreePasswordBuffer( pszDecriptedPassword );
}
#endif