#include <windows.h>
#include <wincrypt.h>
#include <stdio.h>
#include <tchar.h>
#include "crypto2.h"

static TCHAR szLocalPassword[] = _T("a2[@%>-.KGSV");


static BOOL (WINAPI *s_pfCryptAcquireContext)(
  HCRYPTPROV *phProv,
  LPCTSTR pszContainer,
  LPCTSTR pszProvider,
  DWORD dwProvType,
  DWORD dwFlags
);
static BOOL (WINAPI *s_pfCryptGetUserKey)(
  HCRYPTPROV hProv,      
  DWORD dwKeySpec,       
  HCRYPTKEY *phUserKey   
);
static BOOL (WINAPI *s_pfCryptGenKey)(
  HCRYPTPROV hProv,  
  ALG_ID Algid,      
  DWORD dwFlags,     
  HCRYPTKEY *phKey   
);
static BOOL (WINAPI *s_pfCryptReleaseContext)(
  HCRYPTPROV hProv,  
  DWORD dwFlags      
);
static BOOL (WINAPI *s_pfCryptDestroyKey)(
  HCRYPTKEY hKey     
);
static BOOL (WINAPI *s_pfCryptCreateHash)(
  HCRYPTPROV hProv,    
  ALG_ID Algid,        
  HCRYPTKEY hKey,      
  DWORD dwFlags,       
  HCRYPTHASH *phHash   
);
static BOOL (WINAPI *s_pfCryptHashData)(
  HCRYPTHASH hHash,  
  BYTE *pbData,      
  DWORD dwDataLen,   
  DWORD dwFlags      
);
static BOOL (WINAPI *s_pfCryptDeriveKey)(
  HCRYPTPROV hProv,      
  ALG_ID Algid,          
  HCRYPTHASH hBaseData,  
  DWORD dwFlags,         
  HCRYPTKEY *phKey       
);
static BOOL (WINAPI *s_pfCryptEncrypt)(
  HCRYPTKEY hKey,      
  HCRYPTHASH hHash,    
  BOOL Final,          
  DWORD dwFlags,       
  BYTE *pbData,        
  DWORD *pdwDataLen,   
  DWORD dwBufLen       
);
static BOOL (WINAPI *s_pfCryptDestroyHash)(
  HCRYPTHASH hHash       
);
static BOOL (WINAPI *s_pfCryptDecrypt)(
  HCRYPTKEY hKey,    
  HCRYPTHASH hHash,  
  BOOL Final,        
  DWORD dwFlags,     
  BYTE *pbData,      
  DWORD *pdwDataLen  
);



static BOOL LoadCryptoDll()
{
    //static HMODULE s_hCrypt32;
    static HMODULE s_hAdvapi32;

    if( s_hAdvapi32 == NULL )
    {
	s_hAdvapi32 = LoadLibrary( TEXT("advapi32.dll") );
	if( ! s_hAdvapi32 ) return FALSE;
	s_pfCryptAcquireContext = (BOOL (WINAPI *)(HCRYPTPROV *,  LPCTSTR,  LPCTSTR,  DWORD,  DWORD ))GetProcAddress( s_hAdvapi32, "CryptAcquireContextA" );
	s_pfCryptGetUserKey = (BOOL (WINAPI *)(HCRYPTPROV hProv, DWORD ,  HCRYPTKEY *))GetProcAddress( s_hAdvapi32, "CryptGetUserKey" );
	s_pfCryptGenKey = (BOOL (WINAPI *)(HCRYPTPROV, ALG_ID, DWORD, HCRYPTKEY *))GetProcAddress( s_hAdvapi32, "CryptGenKey" );
	s_pfCryptReleaseContext = (BOOL (WINAPI *)(HCRYPTPROV,DWORD ))GetProcAddress( s_hAdvapi32, "CryptReleaseContext" );
	s_pfCryptDestroyKey = (BOOL (WINAPI *)(HCRYPTKEY ))GetProcAddress( s_hAdvapi32, "CryptDestroyKey" );
	s_pfCryptCreateHash = (BOOL (WINAPI *)(HCRYPTPROV, ALG_ID ,HCRYPTKEY ,DWORD ,HCRYPTHASH *))GetProcAddress( s_hAdvapi32, "CryptCreateHash" );
	s_pfCryptHashData = (BOOL (WINAPI *)(HCRYPTHASH, BYTE *, DWORD, DWORD ))GetProcAddress( s_hAdvapi32, "CryptHashData" );
	s_pfCryptDeriveKey = (BOOL (WINAPI *)(HCRYPTPROV, ALG_ID, HCRYPTHASH, DWORD, HCRYPTKEY *))GetProcAddress( s_hAdvapi32, "CryptDeriveKey" );
	s_pfCryptEncrypt = (BOOL (WINAPI *)(HCRYPTKEY, HCRYPTHASH, BOOL, DWORD, BYTE *, DWORD *, DWORD ))GetProcAddress( s_hAdvapi32, "CryptEncrypt" );
	s_pfCryptDestroyHash = (BOOL (WINAPI *)(HCRYPTHASH ))GetProcAddress( s_hAdvapi32, "CryptDestroyHash" );
	s_pfCryptDecrypt = (BOOL (WINAPI *)(HCRYPTKEY, HCRYPTHASH, BOOL, DWORD, BYTE *, DWORD *))GetProcAddress( s_hAdvapi32, "CryptDecrypt" );
	if( ! s_pfCryptAcquireContext	||
	    ! s_pfCryptGetUserKey	||
	    ! s_pfCryptGenKey		||
	    ! s_pfCryptReleaseContext	||
	    ! s_pfCryptDestroyKey	||
	    ! s_pfCryptCreateHash	||
	    ! s_pfCryptHashData		||
	    ! s_pfCryptDeriveKey	||
	    ! s_pfCryptEncrypt		||
	    ! s_pfCryptDestroyHash	||
	    ! s_pfCryptDecrypt )
	{
	    FreeLibrary( s_hAdvapi32 );
	    s_hAdvapi32 = NULL;
	    return FALSE;
	}

    }
    return TRUE;
}



//To ensure that the default cryptographic client is correctly setup an application 
// may call the following function, which returns TRUE if the client is correctly 
// installed and FALSE otherwise: 
static BOOL SetupCryptoClient()
{
    // Ensure that the default cryptographic client is set up.
    HCRYPTPROV hProv;
    HCRYPTKEY hKey;
    static BOOL s_fInitialized = FALSE;

    if( ! LoadCryptoDll() )return FALSE;
    
    if( s_fInitialized == TRUE ) return TRUE;
    s_fInitialized = TRUE;
    
    // Attempt to acquire a handle to the default key container.
    if( !(*s_pfCryptAcquireContext)(&hProv, NULL, MS_DEF_PROV, PROV_RSA_FULL, 0))
    {
	// Some sort of error occured, create default key container.
	if( !(*s_pfCryptAcquireContext)(&hProv, NULL, MS_DEF_PROV, PROV_RSA_FULL, CRYPT_NEWKEYSET))
	{
	    // Error creating key container!
	    return FALSE;
	}
    }
    
    // Attempt to get handle to signature key.
    if( !(*s_pfCryptGetUserKey)(hProv, AT_SIGNATURE, &hKey))
    {
	if( GetLastError() == NTE_NO_KEY)
	{
	    // Create signature key pair.
	    if( !(*s_pfCryptGenKey)(hProv, AT_SIGNATURE, 0, &hKey))
	    {
		// Error during CryptGenKey!
		(*s_pfCryptReleaseContext)(hProv, 0);
		return FALSE;
	    }
	    else
	    {
		(*s_pfCryptDestroyKey)(hKey);
	    }
	}
	else 
	{
	    // Error during CryptGetUserKey!
	    (*s_pfCryptReleaseContext)(hProv, 0);
	    return FALSE;
	}
    }
    
    // Attempt to get handle to exchange key.
    if( !(*s_pfCryptGetUserKey)(hProv,AT_KEYEXCHANGE,&hKey))
    {
	if( GetLastError()==NTE_NO_KEY)
	{
	    // Create key exchange key pair.
	    if( !(*s_pfCryptGenKey)(hProv,AT_KEYEXCHANGE,0,&hKey))
	    {
		// Error during CryptGenKey!
		(*s_pfCryptReleaseContext)(hProv, 0);
		return FALSE;
	    }
	    else
	    {
		(*s_pfCryptDestroyKey)(hKey);
	    }
	}
	else
	{
	    // Error during CryptGetUserKey!
	    (*s_pfCryptReleaseContext)(hProv, 0);
	    return FALSE;
	}
    }
    
    (*s_pfCryptReleaseContext)(hProv, 0);
    return TRUE;
}



/*---------------------------------------------------------------------------
**
**        ENCRYPTSTRING
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --EncryptString--
**  ARGUMENTS
**        Argument1:  LPCTSTR pszIn
**        Argument2: LPCTSTR pbOutEncrypted
**        Argument3: DWORD dwOut (should be >= strlen(pszIn))
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 12/06/2003 
**--------------------------------------------------------------------------*/
BOOL EncryptString( LPCTSTR pszIn, BYTE * pbOutEncrypted, DWORD dwOut )
{
    BOOL bResult = TRUE;    
    HCRYPTPROV hProv = 0;
    HCRYPTKEY hKey = 0;
    HCRYPTKEY hXchgKey = 0;
    HCRYPTHASH hHash = 0;
    DWORD dwLength;

    if( ! SetupCryptoClient() ) return FALSE;
    
    // Get handle to user default provider.
    if( (*s_pfCryptAcquireContext)(&hProv, NULL, NULL, PROV_RSA_FULL, 0))
    {
	// Create hash object.
	if( (*s_pfCryptCreateHash)(hProv, CALG_MD5, 0, 0, &hHash))
	{
	    // Hash password string.
	    dwLength = sizeof(TCHAR)*_tcslen(szLocalPassword);
	    if( (*s_pfCryptHashData)(hHash, (BYTE *)szLocalPassword, dwLength, 0))
	    {
		// Create block cipher session key based on hash of the password.
		if( (*s_pfCryptDeriveKey)(hProv, CALG_RC4, hHash, CRYPT_EXPORTABLE, &hKey))
		{
		    // Determine number of bytes to encrypt at a time.
		    dwLength = sizeof(TCHAR)*(_tcslen(pszIn) + 1);
		    
		    // String to small
		    if( dwLength > dwOut || pbOutEncrypted == NULL )
		    {
			bResult = FALSE;
		    }
		    
		    if( bResult )
		    {
			memcpy(pbOutEncrypted, pszIn, dwLength);
			// Encrypt data
			if( (*s_pfCryptEncrypt)(hKey, 0, TRUE, 0, pbOutEncrypted, &dwLength, dwLength)) 
			{
			    bResult = TRUE;
			}	
			else
			{
			    bResult = FALSE;
			}
		    }
		    (*s_pfCryptDestroyKey)(hKey);  // Release provider handle.
		}
		else
		{
		    // Error during CryptDeriveKey!
		    bResult = FALSE;
		}
	    }
	    else
	    {
		// Error during CryptHashData!
		bResult = FALSE;
	    }
	    (*s_pfCryptDestroyHash)(hHash); // Destroy session key.
	}
	else
	{
	    // Error during CryptCreateHash!
	    bResult = FALSE;
	}
	(*s_pfCryptReleaseContext)(hProv, 0);
    }
    
    return bResult;
}



/*---------------------------------------------------------------------------
**
**        DECRYPTSTRING
**---------------------------------------------------------------------------
**
**  DESCRIPTION
**        --DecryptString--
**  ARGUMENTS
**        Argument1:  BYTE * pbInEncrypted
**        Argument2: DWORD dwInEncrypted; LPTSTR pszOut
**        Argument3: DWORD dwOut
**  RETOUR/RESULTAT
**        BOOL 
**  DIAGNOSTIC
**  IMPLEMENTATION
**  REMARQUE
**  Rev 12/06/2003 
**--------------------------------------------------------------------------*/
BOOL DecryptString( BYTE * pbInEncrypted, DWORD dwInEncrypted, LPTSTR pszOut, DWORD dwOut )
{
    BOOL bResult = TRUE;
    
    HCRYPTPROV hProv = 0;
    HCRYPTKEY hKey = 0;
    HCRYPTKEY hXchgKey = 0;
    HCRYPTHASH hHash = 0;
    DWORD dwLength;

    if( dwInEncrypted > dwOut || pszOut == NULL )
    {
	return FALSE;
    }
    
    if( ! SetupCryptoClient() ) return FALSE;

    // Get handle to user default provider.
    if ((*s_pfCryptAcquireContext)(&hProv, NULL, NULL, PROV_RSA_FULL, 0))
    {
	// Create hash object.
	if ((*s_pfCryptCreateHash)(hProv, CALG_MD5, 0, 0, &hHash))
	{
	    // Hash password string.
	    dwLength = sizeof(TCHAR)*_tcslen(szLocalPassword);
	    if ((*s_pfCryptHashData)(hHash, (BYTE *)szLocalPassword, dwLength, 0))
	    {
		// Create block cipher session key based on hash of the password.
		if ((*s_pfCryptDeriveKey)(hProv, CALG_RC4, hHash, CRYPT_EXPORTABLE, &hKey))
		{
		    dwLength = dwInEncrypted;
		    memcpy( pszOut, pbInEncrypted, dwLength );
		    if (!(*s_pfCryptDecrypt)(hKey, 0, TRUE, 0, (BYTE *)pszOut, &dwLength))
		    {
			bResult = FALSE;
		    }
		    (*s_pfCryptDestroyKey)(hKey);  // Release provider handle.
		}
		else
		{
		    // Error during CryptDeriveKey!
		    bResult = FALSE;
		}
	    }
	    else
	    {
		// Error during CryptHashData!
		bResult = FALSE;
	    }
	    (*s_pfCryptDestroyHash)(hHash); // Destroy session key.
	}
	else
	{
	    // Error during CryptCreateHash!
	    bResult = FALSE;
	}
	(*s_pfCryptReleaseContext)(hProv, 0);
    }
    
    return bResult;
}
